@@ -3,12 +3,13 @@ IPy - class and tools for handling of IPv4 and IPv6 addresses and networks.
 See README file for learn how to use IPy.
 
 Further Information might be available at:
-http://software.inl.fr/trac/trac.cgi/wiki/IPy
+https://github.com/haypo/python-ipy
 """
 
-__version__ = '0.72'
+__version__ = '0.75'
 
 import sys
+import types
 
 # Definition of the Ranges for IPv4 IPs
 # this should include www.iana.org/assignments/ipv4-address-space
@@ -26,44 +27,90 @@ IPv4ranges = {
     }
 
 # Definition of the Ranges for IPv6 IPs
-# see also www.iana.org/assignments/ipv6-address-space,
-# www.iana.org/assignments/ipv6-tla-assignments,
-# www.iana.org/assignments/ipv6-multicast-addresses,
-# www.iana.org/assignments/ipv6-anycast-addresses
+# http://www.iana.org/assignments/ipv6-address-space/
+# http://www.iana.org/assignments/ipv6-unicast-address-assignments/
+# http://www.iana.org/assignments/ipv6-multicast-addresses/
 IPv6ranges = {
-    '00000000'              : 'RESERVED',       # ::/8
-    '00000001'              : 'UNASSIGNED',     # 100::/8
-    '0000001'               : 'NSAP',           # 200::/7
-    '0000010'               : 'IPX',            # 400::/7
-    '0000011'               : 'UNASSIGNED',     # 600::/7
-    '00001'                 : 'UNASSIGNED',     # 800::/5
-    '0001'                  : 'UNASSIGNED',     # 1000::/4
-    '0010000000000000'      : 'RESERVED',       # 2000::/16 Reserved
-    '0010000000000001'      : 'ASSIGNABLE',     # 2001::/16 Sub-TLA Assignments [RFC2450]
-    '00100000000000010000000': 'ASSIGNABLE IANA',  # 2001:0000::/29 - 2001:01F8::/29 IANA
-    '00100000000000010000001': 'ASSIGNABLE APNIC', # 2001:0200::/29 - 2001:03F8::/29 APNIC
-    '00100000000000010000010': 'ASSIGNABLE ARIN',  # 2001:0400::/29 - 2001:05F8::/29 ARIN
-    '00100000000000010000011': 'ASSIGNABLE RIPE',  # 2001:0600::/29 - 2001:07F8::/29 RIPE NCC
-    '0010000000000010'      : '6TO4',           # 2002::/16 "6to4" [RFC3056]
-    '0011111111111110'      : '6BONE',          # 3FFE::/16 6bone Testing [RFC2471]
-    '0011111111111111'      : 'RESERVED',       # 3FFF::/16 Reserved
-    '010'                   : 'GLOBAL-UNICAST', # 4000::/3
-    '011'                   : 'UNASSIGNED',     # 6000::/3
-    '100'                   : 'GEO-UNICAST',    # 8000::/3
-    '101'                   : 'UNASSIGNED',     # A000::/3
-    '110'                   : 'UNASSIGNED',     # C000::/3
-    '1110'                  : 'UNASSIGNED',     # E000::/4
-    '11110'                 : 'UNASSIGNED',     # F000::/5
-    '111110'                : 'UNASSIGNED',     # F800::/6
-    '1111110'               : 'UNASSIGNED',     # FC00::/7
-    '111111100'             : 'UNASSIGNED',     # FE00::/9
-    '1111111010'            : 'LINKLOCAL',      # FE80::/10
-    '1111111011'            : 'SITELOCAL',      # FEC0::/10
-    '11111111'              : 'MULTICAST',      # FF00::/8
-    '0' * 96                : 'IPV4COMP',       # ::/96
-    '0' * 80 + '1' * 16     : 'IPV4MAP',        # ::FFFF:0:0/96
-    '0' * 128               : 'UNSPECIFIED',    # ::/128
-    '0' * 127 + '1'         : 'LOOPBACK'        # ::1/128
+    '00000000'                                      : 'RESERVED',               # ::/8
+    '0' * 96                                        : 'RESERVED',               # ::/96 Formerly IPV4COMP [RFC4291]
+    '0' * 128                                       : 'UNSPECIFIED',            # ::/128
+    '0' * 127 + '1'                                 : 'LOOPBACK',               # ::1/128
+    '0' * 80 + '1' * 16                             : 'IPV4MAP',                # ::ffff:0:0/96
+    '00000000011001001111111110011011' + '0' * 64   : 'WKP46TRANS',             # 0064:ff9b::/96 Well-Known-Prefix [RFC6052]
+    '00000001'                                      : 'UNASSIGNED',             # 0100::/8
+    '0000001'                                       : 'RESERVED',               # 0200::/7 Formerly NSAP [RFC4048]
+    '0000010'                                       : 'RESERVED',               # 0400::/7 Formerly IPX [RFC3513]
+    '0000011'                                       : 'RESERVED',               # 0600::/7
+    '00001'                                         : 'RESERVED',               # 0800::/5
+    '0001'                                          : 'RESERVED',               # 1000::/4
+    '001'                                           : 'GLOBAL-UNICAST',         # 2000::/3 [RFC4291]
+    '00100000000000010000000'                       : 'SPECIALPURPOSE',         # 2001::/23 [RFC4773]
+    '00100000000000010000000000000000'              : 'TEREDO',                 # 2001::/32 [RFC4380]
+    '00100000000000010000000000000010' + '0' * 16   : 'BMWG',                   # 2001:0002::/48 Benchmarking [RFC5180]
+    '0010000000000001000000000001'                  : 'ORCHID',                 # 2001:0010::/28 (Temp until 2014-03-21) [RFC4843]
+    '00100000000000010000001'                       : 'ALLOCATED APNIC',        # 2001:0200::/23
+    '00100000000000010000010'                       : 'ALLOCATED ARIN',         # 2001:0400::/23
+    '00100000000000010000011'                       : 'ALLOCATED RIPE NCC',     # 2001:0600::/23
+    '00100000000000010000100'                       : 'ALLOCATED RIPE NCC',     # 2001:0800::/23
+    '00100000000000010000101'                       : 'ALLOCATED RIPE NCC',     # 2001:0a00::/23
+    '00100000000000010000110'                       : 'ALLOCATED APNIC',        # 2001:0c00::/23
+    '00100000000000010000110110111000'              : 'DOCUMENTATION',          # 2001:0db8::/32 [RFC3849]
+    '00100000000000010000111'                       : 'ALLOCATED APNIC',        # 2001:0e00::/23
+    '00100000000000010001001'                       : 'ALLOCATED LACNIC',       # 2001:1200::/23
+    '00100000000000010001010'                       : 'ALLOCATED RIPE NCC',     # 2001:1400::/23
+    '00100000000000010001011'                       : 'ALLOCATED RIPE NCC',     # 2001:1600::/23
+    '00100000000000010001100'                       : 'ALLOCATED ARIN',         # 2001:1800::/23
+    '00100000000000010001101'                       : 'ALLOCATED RIPE NCC',     # 2001:1a00::/23
+    '0010000000000001000111'                        : 'ALLOCATED RIPE NCC',     # 2001:1c00::/22
+    '00100000000000010010'                          : 'ALLOCATED RIPE NCC',     # 2001:2000::/20
+    '001000000000000100110'                         : 'ALLOCATED RIPE NCC',     # 2001:3000::/21
+    '0010000000000001001110'                        : 'ALLOCATED RIPE NCC',     # 2001:3800::/22
+    '0010000000000001001111'                        : 'RESERVED',               # 2001:3c00::/22 Possible future allocation to RIPE NCC
+    '00100000000000010100000'                       : 'ALLOCATED RIPE NCC',     # 2001:4000::/23
+    '00100000000000010100001'                       : 'ALLOCATED AFRINIC',      # 2001:4200::/23
+    '00100000000000010100010'                       : 'ALLOCATED APNIC',        # 2001:4400::/23
+    '00100000000000010100011'                       : 'ALLOCATED RIPE NCC',     # 2001:4600::/23
+    '00100000000000010100100'                       : 'ALLOCATED ARIN',         # 2001:4800::/23
+    '00100000000000010100101'                       : 'ALLOCATED RIPE NCC',     # 2001:4a00::/23
+    '00100000000000010100110'                       : 'ALLOCATED RIPE NCC',     # 2001:4c00::/23
+    '00100000000000010101'                          : 'ALLOCATED RIPE NCC',     # 2001:5000::/20
+    '0010000000000001100'                           : 'ALLOCATED APNIC',        # 2001:8000::/19
+    '00100000000000011010'                          : 'ALLOCATED APNIC',        # 2001:a000::/20
+    '00100000000000011011'                          : 'ALLOCATED APNIC',        # 2001:b000::/20
+    '0010000000000010'                              : '6TO4',                   # 2002::/16 "6to4" [RFC3056]
+    '001000000000001100'                            : 'ALLOCATED RIPE NCC',     # 2003::/18
+    '001001000000'                                  : 'ALLOCATED APNIC',        # 2400::/12
+    '001001100000'                                  : 'ALLOCATED ARIN',         # 2600::/12
+    '00100110000100000000000'                       : 'ALLOCATED ARIN',         # 2610::/23
+    '00100110001000000000000'                       : 'ALLOCATED ARIN',         # 2620::/23
+    '001010000000'                                  : 'ALLOCATED LACNIC',       # 2800::/12
+    '001010100000'                                  : 'ALLOCATED RIPE NCC',     # 2a00::/12
+    '001011000000'                                  : 'ALLOCATED AFRINIC',      # 2c00::/12
+    '00101101'                                      : 'RESERVED',               # 2d00::/8
+    '0010111'                                       : 'RESERVED',               # 2e00::/7
+    '0011'                                          : 'RESERVED',               # 3000::/4
+    '010'                                           : 'RESERVED',               # 4000::/3
+    '011'                                           : 'RESERVED',               # 6000::/3
+    '100'                                           : 'RESERVED',               # 8000::/3
+    '101'                                           : 'RESERVED',               # a000::/3
+    '110'                                           : 'RESERVED',               # c000::/3
+    '1110'                                          : 'RESERVED',               # e000::/4
+    '11110'                                         : 'RESERVED',               # f000::/5
+    '111110'                                        : 'RESERVED',               # f800::/6
+    '1111110'                                       : 'ULA',                    # fc00::/7 [RFC4193]
+    '111111100'                                     : 'RESERVED',               # fe00::/9
+    '1111111010'                                    : 'LINKLOCAL',              # fe80::/10
+    '1111111011'                                    : 'RESERVED',               # fec0::/10 Formerly SITELOCAL [RFC4291]
+    '11111111'                                      : 'MULTICAST',              # ff00::/8
+    '1111111100000001'                              : 'NODE-LOCAL MULTICAST',   # ff01::/16
+    '1111111100000010'                              : 'LINK-LOCAL MULTICAST',   # ff02::/16
+    '1111111100000100'                              : 'ADMIN-LOCAL MULTICAST',  # ff04::/16
+    '1111111100000101'                              : 'SITE-LOCAL MULTICAST',   # ff05::/16
+    '1111111100001000'                              : 'ORG-LOCAL MULTICAST',    # ff08::/16
+    '1111111100001110'                              : 'GLOBAL MULTICAST',       # ff0e::/16
+    '1111111100001111'                              : 'RESERVED MULTICAST',     # ff0f::/16
+    '111111110011'                                  : 'PREFIX-BASED MULTICAST', # ff30::/12 [RFC3306]
+    '111111110111'                                  : 'RP-EMBEDDED MULTICAST',  # ff70::/12 [RFC3956]
     }
 
 
@@ -90,13 +137,13 @@ class IPint:
         If make_net is True, an IP address will be transformed into the network
         address by applying the specified netmask.
 
-        >>> print IP('127.0.0.0/8')
+        >>> print(IP('127.0.0.0/8'))
         127.0.0.0/8
-        >>> print IP('127.0.0.0/255.0.0.0')
+        >>> print(IP('127.0.0.0/255.0.0.0'))
         127.0.0.0/8
-        >>> print IP('127.0.0.0-127.255.255.255')
+        >>> print(IP('127.0.0.0-127.255.255.255'))
         127.0.0.0/8
-        >>> print IP('127.0.0.1/255.0.0.0', make_net=True)
+        >>> print(IP('127.0.0.1/255.0.0.0', make_net=True))
         127.0.0.0/8
 
         See module documentation for more examples.
@@ -128,7 +175,7 @@ class IPint:
                     raise ValueError("IPv6 Addresses can't be larger than 0xffffffffffffffffffffffffffffffff: %x" % self.ip)
                 prefixlen = 128
             else:
-                raise ValueError, "only IPv4 and IPv6 supported"
+                raise ValueError("only IPv4 and IPv6 supported")
             self._ipversion = ipversion
             self._prefixlen = prefixlen
         # handle IP instance as an parameter
@@ -145,20 +192,19 @@ class IPint:
                 (ip, last) = x
                 (self.ip, parsedVersion) = parseAddress(ip)
                 if parsedVersion != 4:
-                    raise ValueError, "first-last notation only allowed for IPv4"
+                    raise ValueError("first-last notation only allowed for IPv4")
                 (last, lastversion) = parseAddress(last)
                 if lastversion != 4:
-                    raise ValueError, "last address should be IPv4, too"
+                    raise ValueError("last address should be IPv4, too")
                 if last < self.ip:
-                    raise ValueError, "last address should be larger than first"
+                    raise ValueError("last address should be larger than first")
                 size = last - self.ip
                 netbits = _count1Bits(size)
                 # make sure the broadcast is the same as the last ip
                 # otherwise it will return /16 for something like:
                 # 192.168.0.0-192.168.191.255
                 if IP('%s/%s' % (ip, 32-netbits)).broadcast().int() != last:
-                    raise ValueError, \
-                        "the range %s is not on a network boundary." % data
+                    raise ValueError("the range %s is not on a network boundary." % data)
             elif len(x) == 1:
                 x = data.split('/')
                 # if no prefix is given use defaults
@@ -166,7 +212,7 @@ class IPint:
                     ip = x[0]
                     prefixlen = -1
                 elif len(x) > 2:
-                    raise ValueError, "only one '/' allowed in IP Address"
+                    raise ValueError("only one '/' allowed in IP Address")
                 else:
                     (ip, prefixlen) = x
                     if prefixlen.find('.') != -1:
@@ -174,12 +220,12 @@ class IPint:
                         # a.b.c.d/255.255.255.0
                         (netmask, vers) = parseAddress(prefixlen)
                         if vers != 4:
-                            raise ValueError, "netmask must be IPv4"
+                            raise ValueError("netmask must be IPv4")
                         prefixlen = _netmaskToPrefixlen(netmask)
             elif len(x) > 2:
-                raise ValueError, "only one '-' allowed in IP Address"
+                raise ValueError("only one '-' allowed in IP Address")
             else:
-                raise ValueError, "can't parse"
+                raise ValueError("can't parse")
 
             (self.ip, parsedVersion) = parseAddress(ip)
             if ipversion == 0:
@@ -190,7 +236,7 @@ class IPint:
                 elif ipversion == 6:
                     prefixlen = 128 - netbits
                 else:
-                    raise ValueError, "only IPv4 and IPv6 supported"
+                    raise ValueError("only IPv4 and IPv6 supported")
             self._ipversion = ipversion
             self._prefixlen = int(prefixlen)
 
@@ -199,7 +245,7 @@ class IPint:
 
             if not _checkNetaddrWorksWithPrefixlen(self.ip,
             self._prefixlen, self._ipversion):
-                raise ValueError, "%s has invalid prefix length (%s)" % (repr(self), self._prefixlen)
+                raise ValueError("%s has invalid prefix length (%s)" % (repr(self), self._prefixlen))
         else:
             raise TypeError("Unsupported data type: %s" % type(data))
 
@@ -289,7 +335,7 @@ class IPint:
     def strBin(self, wantprefixlen = None):
         """Return a string representation as a binary value.
 
-        >>> print IP('127.0.0.1').strBin()
+        >>> print(IP('127.0.0.1').strBin())
         01111111000000000000000000000001
         """
 
@@ -299,7 +345,7 @@ class IPint:
         elif self._ipversion == 6:
             bits = 128
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
         if self.WantPrefixLen == None and wantprefixlen == None:
             wantprefixlen = 0
@@ -354,9 +400,9 @@ class IPint:
     def strNormal(self, wantprefixlen = None):
         """Return a string representation in the usual format.
 
-        >>> print IP('127.0.0.1').strNormal()
+        >>> print(IP('127.0.0.1').strNormal())
         127.0.0.1
-        >>> print IP('2001:0658:022a:cafe:0200::1').strNormal()
+        >>> print(IP('2001:0658:022a:cafe:0200::1').strNormal())
         2001:658:22a:cafe:200:0:0:1
         """
 
@@ -368,7 +414,7 @@ class IPint:
         elif self._ipversion == 6:
             ret = ':'.join([hex(x)[2:] for x in [int(x, 16) for x in self.strFullsize(0).split(':')]])
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
 
 
@@ -377,9 +423,9 @@ class IPint:
     def strFullsize(self, wantprefixlen = None):
         """Return a string representation in the non-mangled format.
 
-        >>> print IP('127.0.0.1').strFullsize()
+        >>> print(IP('127.0.0.1').strFullsize())
         127.0.0.1
-        >>> print IP('2001:0658:022a:cafe:0200::1').strFullsize()
+        >>> print(IP('2001:0658:022a:cafe:0200::1').strFullsize())
         2001:0658:022a:cafe:0200:0000:0000:0001
         """
 
@@ -408,9 +454,9 @@ class IPint:
     def strDec(self, wantprefixlen = None):
         """Return a string representation in decimal format.
 
-        >>> print IP('127.0.0.1').strDec()
+        >>> print(IP('127.0.0.1').strDec())
         2130706433
-        >>> print IP('2001:0658:022a:cafe:0200::1').strDec()
+        >>> print(IP('2001:0658:022a:cafe:0200::1').strDec())
         42540616829182469433547762482097946625
         """
 
@@ -425,16 +471,16 @@ class IPint:
     def iptype(self):
         """Return a description of the IP type ('PRIVATE', 'RESERVERD', etc).
 
-        >>> print IP('127.0.0.1').iptype()
+        >>> print(IP('127.0.0.1').iptype())
         PRIVATE
-        >>> print IP('192.168.1.1').iptype()
+        >>> print(IP('192.168.1.1').iptype())
         PRIVATE
-        >>> print IP('195.185.1.2').iptype()
+        >>> print(IP('195.185.1.2').iptype())
         PUBLIC
-        >>> print IP('::1').iptype()
+        >>> print(IP('::1').iptype())
         LOOPBACK
-        >>> print IP('2001:0658:022a:cafe:0200::1').iptype()
-        ASSIGNABLE RIPE
+        >>> print(IP('2001:0658:022a:cafe:0200::1').iptype())
+        ALLOCATED RIPE NCC
 
         The type information for IPv6 is out of sync with reality.
         """
@@ -446,11 +492,11 @@ class IPint:
         elif self._ipversion == 6:
             iprange = IPv6ranges
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
         bits = self.strBin()
         for i in xrange(len(bits), 0, -1):
-            if iprange.has_key(bits[:i]):
+            if bits[:i] in iprange:
                 return iprange[bits[:i]]
         return "unknown"
 
@@ -468,7 +514,7 @@ class IPint:
         elif self._ipversion == 6:
             locallen = 128 - self._prefixlen
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
         return ((2 ** self._prefixlen) - 1) << locallen
 
@@ -476,9 +522,9 @@ class IPint:
     def strNetmask(self):
         """Return netmask as an string. Mostly useful for IPv6.
 
-        >>> print IP('195.185.0.0/16').strNetmask()
+        >>> print(IP('195.185.0.0/16').strNetmask())
         255.255.0.0
-        >>> print IP('2001:0658:022a:cafe::0/64').strNetmask()
+        >>> print(IP('2001:0658:022a:cafe::0/64').strNetmask())
         /64
         """
 
@@ -490,14 +536,14 @@ class IPint:
             locallen = 128 - self._prefixlen
             return "/%d" % self._prefixlen
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
     def len(self):
         """Return the length of a subnet.
 
-        >>> print IP('195.185.1.0/28').len()
+        >>> print(IP('195.185.1.0/28').len())
         16
-        >>> print IP('195.185.1.0/24').len()
+        >>> print(IP('195.185.1.0/24').len())
         256
         """
 
@@ -506,7 +552,7 @@ class IPint:
         elif self._ipversion == 6:
             locallen = 128 - self._prefixlen
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
         return 2 ** locallen
 
@@ -517,7 +563,7 @@ class IPint:
         0.0.0.0/0, the __len__() of the object becomes 0, which is used
         as the boolean value of the object.
         """
-        return 1
+        return True
 
 
     def __len__(self):
@@ -537,7 +583,7 @@ class IPint:
 
         >>> ip=IP('127.0.0.0/30')
         >>> for x in ip:
-        ...  print repr(x)
+        ...  print(repr(x))
         ...
         IP('127.0.0.0')
         IP('127.0.0.1')
@@ -573,18 +619,18 @@ class IPint:
         >>> IP('195.185.1.1').strHex()
         '0xc3b90101'
         >>> 0xC3B90101 in IP('195.185.1.0/24')
-        1
+        True
         >>> '127.0.0.1' in IP('127.0.0.0/24')
-        1
+        True
         >>> IP('127.0.0.0/24') in IP('127.0.0.0/25')
-        0
+        False
         """
 
         item = IP(item)
         if item.ip >= self.ip and item.ip < self.ip + self.len() - item.len() + 1:
-            return 1
+            return True
         else:
-            return 0
+            return False
 
 
     def overlaps(self, item):
@@ -627,7 +673,7 @@ class IPint:
         to an identical Object (without the wantprefixlen stuff - see
         module docstring.
 
-        >>> print repr(IP('10.0.0.0/24'))
+        >>> print(repr(IP('10.0.0.0/24')))
         IP('10.0.0.0/24')
         """
 
@@ -750,8 +796,17 @@ class IP(IPint):
         >>> IP('10.0.0.0/8').netmask()
         IP('255.0.0.0')
          """
-        return IP(IPint.netmask(self))
+        return IP(IPint.netmask(self), ipversion=self._ipversion)
 
+    def _getIPv4Map(self):
+        if self._ipversion != 6:
+            return None
+        if (self.ip >> 32) != 0xffff:
+            return None
+        ipv4 = self.ip & 0xffffffff
+        if self._prefixlen != 128:
+            ipv4 = '%s/%s' % (ipv4, 32-(128-self._prefixlen))
+        return IP(ipv4, ipversion=4)
 
     def reverseNames(self):
         """Return a list with values forming the reverse lookup.
@@ -793,20 +848,21 @@ class IP(IPint):
                     ret.append(self[i].reverseName()[6:])
             return ret
         elif self._ipversion == 6:
+            ipv4 = self._getIPv4Map()
+            if ipv4 is not None:
+                return ipv4.reverseNames()
             s = hex(self.ip)[2:].lower()
             if s[-1] == 'l':
                 s = s[:-1]
             if self._prefixlen % 4 != 0:
-                raise NotImplementedError, "can't create IPv6 reverse names at sub nibble level"
+                raise NotImplementedError("can't create IPv6 reverse names at sub nibble level")
             s = list(s)
             s.reverse()
             s = '.'.join(s)
             first_nibble_index = int(32 - (self._prefixlen // 4)) * 2
             return ["%s.ip6.arpa." % s[first_nibble_index:]]
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
-
-
+            raise ValueError("only IPv4 and IPv6 supported")
 
     def reverseName(self):
         """Return the value for reverse lookup/PTR records as RFC 2317 look alike.
@@ -815,9 +871,9 @@ class IP(IPint):
         for /23. Do not use it. Better set up a zone for every
         address. See reverseName for a way to achieve that.
 
-        >>> print IP('195.185.1.1').reverseName()
+        >>> print(IP('195.185.1.1').reverseName())
         1.1.185.195.in-addr.arpa.
-        >>> print IP('195.185.1.0/28').reverseName()
+        >>> print(IP('195.185.1.0/28').reverseName())
         0-15.1.185.195.in-addr.arpa.
         >>> IP('::1:2').reverseName()
         '2.0.0.0.1.ip6.arpa.'
@@ -840,6 +896,9 @@ class IP(IPint):
             return "%s%s.in-addr.arpa." % (nibblepart, s)
 
         elif self._ipversion == 6:
+            ipv4 = self._getIPv4Map()
+            if ipv4 is not None:
+                return ipv4.reverseName()
             s = hex(self.ip)[2:].lower()
             if s[-1] == 'l':
                 s = s[:-1]
@@ -856,7 +915,7 @@ class IP(IPint):
             first_nibble_index = int(32 - (self._prefixlen // 4)) * 2
             return "%s%s.ip6.arpa." % (nibblepart, s[first_nibble_index:])
         else:
-            raise ValueError, "only IPv4 and IPv6 supported"
+            raise ValueError("only IPv4 and IPv6 supported")
 
     def make_net(self, netmask):
         """Transform a single IP address into a network specification by
@@ -864,11 +923,11 @@ class IP(IPint):
 
         Returns a new IP instance.
 
-        >>> print IP('127.0.0.1').make_net('255.0.0.0')
+        >>> print(IP('127.0.0.1').make_net('255.0.0.0'))
         127.0.0.0/8
         """
         if '/' in str(netmask):
-            raise ValueError, "invalid netmask (%s)" % netmask
+            raise ValueError("invalid netmask (%s)" % netmask)
         return IP('%s/%s' % (self, netmask), make_net=True)
 
     def __getitem__(self, key):
@@ -876,15 +935,15 @@ class IP(IPint):
 
         >>> ip=IP('127.0.0.0/30')
         >>> for x in ip:
-        ...  print str(x)
+        ...  print(str(x))
         ...
         127.0.0.0
         127.0.0.1
         127.0.0.2
         127.0.0.3
-        >>> print str(ip[2])
+        >>> print(str(ip[2]))
         127.0.0.2
-        >>> print str(ip[-1])
+        >>> print(str(ip[-1]))
         127.0.0.3
         """
         return IP(IPint.__getitem__(self, key))
@@ -901,11 +960,11 @@ class IP(IPint):
     def __add__(self, other):
         """Emulate numeric objects through network aggregation"""
         if self.prefixlen() != other.prefixlen():
-            raise ValueError, "Only networks with the same prefixlen can be added."
-        if self.prefixlen < 1:
-            raise ValueError, "Networks with a prefixlen longer than /1 can't be added."
+            raise ValueError("Only networks with the same prefixlen can be added.")
+        if self.prefixlen() < 1:
+            raise ValueError("Networks with a prefixlen longer than /1 can't be added.")
         if self.version() != other.version():
-            raise ValueError, "Only networks with the same IP version can be added."
+            raise ValueError("Only networks with the same IP version can be added.")
         if self > other:
             # fixed by Skinny Puppy <skin_pup-IPy@happypoo.com>
             return other.__add__(self)
@@ -919,50 +978,50 @@ def _parseAddressIPv6(ipstr):
     """
     Internal function used by parseAddress() to parse IPv6 address with ':'.
 
-    >>> print _parseAddressIPv6('::')
+    >>> print(_parseAddressIPv6('::'))
     0
-    >>> print _parseAddressIPv6('::1')
+    >>> print(_parseAddressIPv6('::1'))
     1
-    >>> print _parseAddressIPv6('0:0:0:0:0:0:0:1')
+    >>> print(_parseAddressIPv6('0:0:0:0:0:0:0:1'))
     1
-    >>> print _parseAddressIPv6('0:0:0::0:0:1')
+    >>> print(_parseAddressIPv6('0:0:0::0:0:1'))
     1
-    >>> print _parseAddressIPv6('0:0:0:0:0:0:0:0')
+    >>> print(_parseAddressIPv6('0:0:0:0:0:0:0:0'))
     0
-    >>> print _parseAddressIPv6('0:0:0::0:0:0')
+    >>> print(_parseAddressIPv6('0:0:0::0:0:0'))
     0
 
-    >>> print _parseAddressIPv6('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210')
+    >>> print(_parseAddressIPv6('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210'))
     338770000845734292534325025077361652240
-    >>> print _parseAddressIPv6('1080:0000:0000:0000:0008:0800:200C:417A')
+    >>> print(_parseAddressIPv6('1080:0000:0000:0000:0008:0800:200C:417A'))
     21932261930451111902915077091070067066
-    >>> print _parseAddressIPv6('1080:0:0:0:8:800:200C:417A')
+    >>> print(_parseAddressIPv6('1080:0:0:0:8:800:200C:417A'))
     21932261930451111902915077091070067066
-    >>> print _parseAddressIPv6('1080:0::8:800:200C:417A')
+    >>> print(_parseAddressIPv6('1080:0::8:800:200C:417A'))
     21932261930451111902915077091070067066
-    >>> print _parseAddressIPv6('1080::8:800:200C:417A')
+    >>> print(_parseAddressIPv6('1080::8:800:200C:417A'))
     21932261930451111902915077091070067066
-    >>> print _parseAddressIPv6('FF01:0:0:0:0:0:0:43')
+    >>> print(_parseAddressIPv6('FF01:0:0:0:0:0:0:43'))
     338958331222012082418099330867817087043
-    >>> print _parseAddressIPv6('FF01:0:0::0:0:43')
+    >>> print(_parseAddressIPv6('FF01:0:0::0:0:43'))
     338958331222012082418099330867817087043
-    >>> print _parseAddressIPv6('FF01::43')
+    >>> print(_parseAddressIPv6('FF01::43'))
     338958331222012082418099330867817087043
-    >>> print _parseAddressIPv6('0:0:0:0:0:0:13.1.68.3')
+    >>> print(_parseAddressIPv6('0:0:0:0:0:0:13.1.68.3'))
     218186755
-    >>> print _parseAddressIPv6('::13.1.68.3')
+    >>> print(_parseAddressIPv6('::13.1.68.3'))
     218186755
-    >>> print _parseAddressIPv6('0:0:0:0:0:FFFF:129.144.52.38')
+    >>> print(_parseAddressIPv6('0:0:0:0:0:FFFF:129.144.52.38'))
     281472855454758
-    >>> print _parseAddressIPv6('::FFFF:129.144.52.38')
+    >>> print(_parseAddressIPv6('::FFFF:129.144.52.38'))
     281472855454758
-    >>> print _parseAddressIPv6('1080:0:0:0:8:800:200C:417A')
+    >>> print(_parseAddressIPv6('1080:0:0:0:8:800:200C:417A'))
     21932261930451111902915077091070067066
-    >>> print _parseAddressIPv6('1080::8:800:200C:417A')
+    >>> print(_parseAddressIPv6('1080::8:800:200C:417A'))
     21932261930451111902915077091070067066
-    >>> print _parseAddressIPv6('::1:2:3:4:5:6')
+    >>> print(_parseAddressIPv6('::1:2:3:4:5:6'))
     1208962713947218704138246
-    >>> print _parseAddressIPv6('1:2:3:4:5:6::')
+    >>> print(_parseAddressIPv6('1:2:3:4:5:6::'))
     5192455318486707404433266432802816
     """
 
@@ -1042,36 +1101,38 @@ def parseAddress(ipstr):
 
     Following address formats are recognized:
 
-    >>> parseAddress('0x0123456789abcdef')           # IPv4 if <= 0xffffffff else IPv6
-    (81985529216486895L, 6)
-    >>> parseAddress('123.123.123.123')              # IPv4
-    (2071690107L, 4)
-    >>> parseAddress('123.123')                      # 0-padded IPv4
-    (2071658496L, 4)
-    >>> parseAddress('1080:0000:0000:0000:0008:0800:200C:417A')
-    (21932261930451111902915077091070067066L, 6)
-    >>> parseAddress('1080:0:0:0:8:800:200C:417A')
-    (21932261930451111902915077091070067066L, 6)
-    >>> parseAddress('1080:0::8:800:200C:417A')
-    (21932261930451111902915077091070067066L, 6)
-    >>> parseAddress('::1')
-    (1, 6)
-    >>> parseAddress('::')
-    (0, 6)
-    >>> ip, version = parseAddress('0:0:0:0:0:FFFF:129.144.52.38')
-    >>> print "%s (IPv%s)" % (ip, version)
+    >>> def testParseAddress(address):
+    ...     ip, version = parseAddress(address)
+    ...     print(("%s (IPv%s)" % (ip, version)))
+    ...
+    >>> testParseAddress('0x0123456789abcdef')           # IPv4 if <= 0xffffffff else IPv6
+    81985529216486895 (IPv6)
+    >>> testParseAddress('123.123.123.123')              # IPv4
+    2071690107 (IPv4)
+    >>> testParseAddress('123.123')                      # 0-padded IPv4
+    2071658496 (IPv4)
+    >>> testParseAddress('1080:0000:0000:0000:0008:0800:200C:417A')
+    21932261930451111902915077091070067066 (IPv6)
+    >>> testParseAddress('1080:0:0:0:8:800:200C:417A')
+    21932261930451111902915077091070067066 (IPv6)
+    >>> testParseAddress('1080:0::8:800:200C:417A')
+    21932261930451111902915077091070067066 (IPv6)
+    >>> testParseAddress('::1')
+    1 (IPv6)
+    >>> testParseAddress('::')
+    0 (IPv6)
+    >>> testParseAddress('0:0:0:0:0:FFFF:129.144.52.38')
     281472855454758 (IPv6)
-    >>> parseAddress('::13.1.68.3')
-    (218186755, 6)
-    >>> ip, version = parseAddress('::FFFF:129.144.52.38')
-    >>> print "%s (IPv%s)" % (ip, version)
+    >>> testParseAddress('::13.1.68.3')
+    218186755 (IPv6)
+    >>> testParseAddress('::FFFF:129.144.52.38')
     281472855454758 (IPv6)
     """
 
     if ipstr.startswith('0x'):
         ret = long(ipstr[2:], 16)
-        if ret > 0xffffffffffffffffffffffffffffffffL:
-            raise ValueError, "%r: IP Address can't be bigger than 2^128" % (ipstr)
+        if ret > 0xffffffffffffffffffffffffffffffff:
+            raise ValueError("%r: IP Address can't be bigger than 2^128" % (ipstr))
         if ret < 0x100000000:
             return (ret, 4)
         else:
@@ -1088,12 +1149,12 @@ def parseAddress(ipstr):
         # assume IPv4  ('127' gets interpreted as '127.0.0.0')
         bytes = ipstr.split('.')
         if len(bytes) > 4:
-            raise ValueError, "IPv4 Address with more than 4 bytes"
+            raise ValueError("IPv4 Address with more than 4 bytes")
         bytes += ['0'] * (4 - len(bytes))
         bytes = [long(x) for x in bytes]
         for x in bytes:
             if x > 255 or x < 0:
-                raise ValueError, "%r: single byte must be 0 <= byte < 256" % (ipstr)
+                raise ValueError("%r: single byte must be 0 <= byte < 256" % (ipstr))
         return ((bytes[0] << 24) + (bytes[1] << 16) + (bytes[2] << 8) + bytes[3], 4)
 
     else:
@@ -1101,9 +1162,9 @@ def parseAddress(ipstr):
         # this ony works for numbers > 255 ... others
         # will be interpreted as IPv4 first byte
         ret = long(ipstr, 10)
-        if ret > 0xffffffffffffffffffffffffffffffffL:
-            raise ValueError, "IP Address can't be bigger than 2^128"
-        if ret <= 0xffffffffL:
+        if ret > 0xffffffffffffffffffffffffffffffff:
+            raise ValueError("IP Address can't be bigger than 2^128")
+        if ret <= 0xffffffff:
             return (ret, 4)
         else:
             return (ret, 6)
@@ -1116,19 +1177,19 @@ def intToIp(ip, version):
     ip = long(ip)
 
     if ip < 0:
-        raise ValueError, "IPs can't be negative: %d" % (ip)
+        raise ValueError("IPs can't be negative: %d" % (ip))
 
     ret = ''
     if version == 4:
-        if ip > 0xffffffffL:
-            raise ValueError, "IPv4 Addresses can't be larger than 0xffffffff: %s" % (hex(ip))
+        if ip > 0xffffffff:
+            raise ValueError("IPv4 Addresses can't be larger than 0xffffffff: %s" % (hex(ip)))
         for l in xrange(4):
-            ret = str(ip & 0xffL) + '.' + ret
+            ret = str(ip & 0xff) + '.' + ret
             ip = ip >> 8
         ret = ret[:-1]
     elif version == 6:
-        if ip > 0xffffffffffffffffffffffffffffffffL:
-            raise ValueError, "IPv6 Addresses can't be larger than 0xffffffffffffffffffffffffffffffff: %s" % (hex(ip))
+        if ip > 0xffffffffffffffffffffffffffffffff:
+            raise ValueError("IPv6 Addresses can't be larger than 0xffffffffffffffffffffffffffffffff: %s" % (hex(ip)))
         if sys.hexversion >= 0x03000000:
             # Remove "0x" prefix
             l = hex(ip)[2:]
@@ -1142,7 +1203,7 @@ def intToIp(ip, version):
                 ret = ':' + ret
         ret = ret[1:]
     else:
-        raise ValueError, "only IPv4 and IPv6 supported"
+        raise ValueError("only IPv4 and IPv6 supported")
 
     return ret
 
@@ -1157,7 +1218,7 @@ def _ipVersionToLen(version):
     Traceback (most recent call last):
       File "<stdin>", line 1, in ?
       File "IPy.py", line 1076, in _ipVersionToLen
-        raise ValueError, "only IPv4 and IPv6 supported"
+        raise ValueError("only IPv4 and IPv6 supported")
     ValueError: only IPv4 and IPv6 supported
     """
 
@@ -1166,7 +1227,7 @@ def _ipVersionToLen(version):
     elif version == 6:
         return 128
     else:
-        raise ValueError, "only IPv4 and IPv6 supported"
+        raise ValueError("only IPv4 and IPv6 supported")
 
 
 def _countFollowingZeros(l):
@@ -1188,15 +1249,12 @@ def _intToBin(val):
     """Return the binary representation of an integer as string."""
 
     if val < 0:
-        raise ValueError, "Only positive values allowed"
+        raise ValueError("Only positive values allowed")
     s = hex(val).lower()
     ret = ''
     if s[-1] == 'l':
         s = s[:-1]
     for x in s[2:]:
-        if __debug__:
-            if not _BitTable.has_key(x):
-                raise AssertionError, "hex() returned strange result"
         ret += _BitTable[x]
     # remove leading zeros
     while ret[0] == '0' and len(ret) > 1:
@@ -1217,7 +1275,7 @@ def _count0Bits(num):
     # this could be so easy if _count1Bits(~long(num)) would work as excepted
     num = long(num)
     if num < 0:
-        raise ValueError, "Only positive Numbers please: %s" % (num)
+        raise ValueError("Only positive Numbers please: %s" % (num))
     ret = 0
     while num > 0:
         if num & 1 == 1:
@@ -1274,17 +1332,14 @@ def _checkNetmask(netmask, masklen):
     # now check if the rest consists only of ones
     while bits > 0:
         if (num & 1) == 0:
-            raise ValueError, "Netmask %s can't be expressed as an prefix." % (hex(netmask))
+            raise ValueError("Netmask %s can't be expressed as an prefix." % (hex(netmask)))
         num = num >> 1
         bits -= 1
 
 
 def _checkNetaddrWorksWithPrefixlen(net, prefixlen, version):
     """Check if a base addess of a network is compatible with a prefixlen"""
-    if net & _prefixlenToNetmask(prefixlen, version) == net:
-        return 1
-    else:
-        return 0
+    return (net & _prefixlenToNetmask(prefixlen, version) == net)
 
 
 def _netmaskToPrefixlen(netmask):
@@ -1309,7 +1364,7 @@ def _prefixlenToNetmask(prefixlen, version):
     if prefixlen == 0:
         return 0
     elif prefixlen < 0:
-        raise ValueError, "Prefixlen must be > 0"
+        raise ValueError("Prefixlen must be > 0")
     return ((2<<prefixlen-1)-1) << (_ipVersionToLen(version) - prefixlen)
 
 
@@ -1320,3 +1375,4 @@ if __name__ == "__main__":
         import sys
         sys.exit(1)
 
+
